/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteMediaPlayerProxy.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "LayerHostingContext.h"
#if ENABLE(ENCRYPTED_MEDIA)
#include "RemoteCDMInstanceIdentifier.h"
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
#include "RemoteLegacyCDMSessionIdentifier.h"
#endif
#include "RemoteMediaPlayerConfiguration.h"
#include "RemoteMediaPlayerProxyMessages.h"
#if ENABLE(MEDIA_SOURCE)
#include "RemoteMediaSourceIdentifier.h"
#endif
#include "RemoteVideoFrameProxy.h"
#include "SandboxExtension.h"
#include "TrackPrivateRemoteIdentifier.h"
#include "WebCoreArgumentCoders.h"
#if PLATFORM(COCOA)
#include <WebCore/ColorSpace.h>
#endif
#include <WebCore/ContentType.h>
#if PLATFORM(COCOA)
#include <WebCore/FloatSize.h>
#endif
#include <WebCore/InbandTextTrackPrivate.h>
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
#include <WebCore/MediaPlaybackTargetContext.h>
#endif
#include <WebCore/MediaPlayer.h>
#include <WebCore/MediaPlayerEnums.h>
#include <WebCore/PlatformScreen.h>
#include <WebCore/SecurityOriginData.h>
#include <optional>
#if PLATFORM(COCOA)
#include <wtf/MachSendRight.h>
#endif
#include <wtf/MediaTime.h>
#include <wtf/MonotonicTime.h>
#include <wtf/URLHash.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace RemoteMediaPlayerProxy {

void PrepareForPlayback::callReply(IPC::Decoder& decoder, CompletionHandler<void(std::optional<WebKit::LayerHostingContextID>&&)>&& completionHandler)
{
    std::optional<std::optional<WebKit::LayerHostingContextID>> inlineLayerHostingContextId;
    decoder >> inlineLayerHostingContextId;
    if (!inlineLayerHostingContextId) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*inlineLayerHostingContextId));
}

void PrepareForPlayback::cancelReply(CompletionHandler<void(std::optional<WebKit::LayerHostingContextID>&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<std::optional<WebKit::LayerHostingContextID>>::create());
}

void PrepareForPlayback::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<WebKit::LayerHostingContextID>& inlineLayerHostingContextId)
{
    encoder.get() << inlineLayerHostingContextId;
    connection.sendSyncReply(WTFMove(encoder));
}

void Load::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebKit::RemoteMediaPlayerConfiguration&&)>&& completionHandler)
{
    std::optional<WebKit::RemoteMediaPlayerConfiguration> playerConfiguration;
    decoder >> playerConfiguration;
    if (!playerConfiguration) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*playerConfiguration));
}

void Load::cancelReply(CompletionHandler<void(WebKit::RemoteMediaPlayerConfiguration&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebKit::RemoteMediaPlayerConfiguration>::create());
}

void Load::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::RemoteMediaPlayerConfiguration& playerConfiguration)
{
    encoder.get() << playerConfiguration;
    connection.sendSyncReply(WTFMove(encoder));
}

#if ENABLE(MEDIA_SOURCE)

void LoadMediaSource::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebKit::RemoteMediaPlayerConfiguration&&)>&& completionHandler)
{
    std::optional<WebKit::RemoteMediaPlayerConfiguration> playerConfiguration;
    decoder >> playerConfiguration;
    if (!playerConfiguration) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*playerConfiguration));
}

void LoadMediaSource::cancelReply(CompletionHandler<void(WebKit::RemoteMediaPlayerConfiguration&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebKit::RemoteMediaPlayerConfiguration>::create());
}

void LoadMediaSource::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::RemoteMediaPlayerConfiguration& playerConfiguration)
{
    encoder.get() << playerConfiguration;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

void DidLoadingProgress::callReply(IPC::Decoder& decoder, CompletionHandler<void(bool&&)>&& completionHandler)
{
    std::optional<bool> flag;
    decoder >> flag;
    if (!flag) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*flag));
}

void DidLoadingProgress::cancelReply(CompletionHandler<void(bool&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<bool>::create());
}

void DidLoadingProgress::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool flag)
{
    encoder.get() << flag;
    connection.sendSyncReply(WTFMove(encoder));
}

void PerformTaskAtMediaTime::callReply(IPC::Decoder& decoder, CompletionHandler<void(std::optional<MediaTime>&&, std::optional<MonotonicTime>&&)>&& completionHandler)
{
    std::optional<std::optional<MediaTime>> mediaTime;
    decoder >> mediaTime;
    if (!mediaTime) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    std::optional<std::optional<MonotonicTime>> monotonicTime;
    decoder >> monotonicTime;
    if (!monotonicTime) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*mediaTime), WTFMove(*monotonicTime));
}

void PerformTaskAtMediaTime::cancelReply(CompletionHandler<void(std::optional<MediaTime>&&, std::optional<MonotonicTime>&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<std::optional<MediaTime>>::create(), IPC::AsyncReplyError<std::optional<MonotonicTime>>::create());
}

void PerformTaskAtMediaTime::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<MediaTime>& mediaTime, const std::optional<MonotonicTime>& monotonicTime)
{
    encoder.get() << mediaTime;
    encoder.get() << monotonicTime;
    connection.sendSyncReply(WTFMove(encoder));
}

void WouldTaintOrigin::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<bool>& wouldTaint)
{
    encoder.get() << wouldTaint;
    connection.sendSyncReply(WTFMove(encoder));
}

#if PLATFORM(IOS_FAMILY)

void ErrorLog::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const String& errorLog)
{
    encoder.get() << errorLog;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

#if PLATFORM(IOS_FAMILY)

void AccessLog::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const String& accessLog)
{
    encoder.get() << accessLog;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

#if PLATFORM(COCOA)

void NativeImageForCurrentTime::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<MachSendRight>& sendRight, const WebCore::DestinationColorSpace& colorSpace)
{
    encoder.get() << sendRight;
    encoder.get() << colorSpace;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

#if PLATFORM(COCOA)

void ColorSpace::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebCore::DestinationColorSpace& colorSpace)
{
    encoder.get() << colorSpace;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

void VideoFrameForCurrentTimeIfChanged::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<WebKit::RemoteVideoFrameProxy::Properties>& videoFrame, bool changed)
{
    encoder.get() << videoFrame;
    encoder.get() << changed;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaPlayerProxy

} // namespace Messages

namespace WebKit {

void RemoteMediaPlayerProxy::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::PrepareForPlayback::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaPlayerProxy::PrepareForPlayback>(connection, decoder, this, &RemoteMediaPlayerProxy::prepareForPlayback);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::Load::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaPlayerProxy::Load>(connection, decoder, this, &RemoteMediaPlayerProxy::load);
#if ENABLE(MEDIA_SOURCE)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::LoadMediaSource::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaPlayerProxy::LoadMediaSource>(connection, decoder, this, &RemoteMediaPlayerProxy::loadMediaSource);
#endif
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::CancelLoad::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::CancelLoad>(connection, decoder, this, &RemoteMediaPlayerProxy::cancelLoad);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::PrepareToPlay::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::PrepareToPlay>(connection, decoder, this, &RemoteMediaPlayerProxy::prepareToPlay);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::Play::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::Play>(connection, decoder, this, &RemoteMediaPlayerProxy::play);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::Pause::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::Pause>(connection, decoder, this, &RemoteMediaPlayerProxy::pause);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetVolume::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetVolume>(connection, decoder, this, &RemoteMediaPlayerProxy::setVolume);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetMuted::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetMuted>(connection, decoder, this, &RemoteMediaPlayerProxy::setMuted);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::Seek::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::Seek>(connection, decoder, this, &RemoteMediaPlayerProxy::seek);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SeekWithTolerance::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SeekWithTolerance>(connection, decoder, this, &RemoteMediaPlayerProxy::seekWithTolerance);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetPreload::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetPreload>(connection, decoder, this, &RemoteMediaPlayerProxy::setPreload);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetPrivateBrowsingMode::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetPrivateBrowsingMode>(connection, decoder, this, &RemoteMediaPlayerProxy::setPrivateBrowsingMode);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetPreservesPitch::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetPreservesPitch>(connection, decoder, this, &RemoteMediaPlayerProxy::setPreservesPitch);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetPitchCorrectionAlgorithm::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetPitchCorrectionAlgorithm>(connection, decoder, this, &RemoteMediaPlayerProxy::setPitchCorrectionAlgorithm);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::PrepareForRendering::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::PrepareForRendering>(connection, decoder, this, &RemoteMediaPlayerProxy::prepareForRendering);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetPageIsVisible::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetPageIsVisible>(connection, decoder, this, &RemoteMediaPlayerProxy::setPageIsVisible);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetShouldMaintainAspectRatio::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetShouldMaintainAspectRatio>(connection, decoder, this, &RemoteMediaPlayerProxy::setShouldMaintainAspectRatio);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::AcceleratedRenderingStateChanged::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::AcceleratedRenderingStateChanged>(connection, decoder, this, &RemoteMediaPlayerProxy::acceleratedRenderingStateChanged);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetShouldDisableSleep::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetShouldDisableSleep>(connection, decoder, this, &RemoteMediaPlayerProxy::setShouldDisableSleep);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetRate::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetRate>(connection, decoder, this, &RemoteMediaPlayerProxy::setRate);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::DidLoadingProgress::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaPlayerProxy::DidLoadingProgress>(connection, decoder, this, &RemoteMediaPlayerProxy::didLoadingProgress);
#if ENABLE(VIDEO_PRESENTATION_MODE)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::UpdateVideoFullscreenInlineImage::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::UpdateVideoFullscreenInlineImage>(connection, decoder, this, &RemoteMediaPlayerProxy::updateVideoFullscreenInlineImage);
#endif
#if ENABLE(VIDEO_PRESENTATION_MODE)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetVideoFullscreenMode::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetVideoFullscreenMode>(connection, decoder, this, &RemoteMediaPlayerProxy::setVideoFullscreenMode);
#endif
#if ENABLE(VIDEO_PRESENTATION_MODE)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetVideoFullscreenGravity::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetVideoFullscreenGravity>(connection, decoder, this, &RemoteMediaPlayerProxy::setVideoFullscreenGravity);
#endif
#if ENABLE(VIDEO_PRESENTATION_MODE)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::VideoFullscreenStandbyChanged::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::VideoFullscreenStandbyChanged>(connection, decoder, this, &RemoteMediaPlayerProxy::videoFullscreenStandbyChanged);
#endif
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetBufferingPolicy::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetBufferingPolicy>(connection, decoder, this, &RemoteMediaPlayerProxy::setBufferingPolicy);
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetVideoInlineSizeFenced::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetVideoInlineSizeFenced>(connection, decoder, this, &RemoteMediaPlayerProxy::setVideoInlineSizeFenced);
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetWirelessVideoPlaybackDisabled::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetWirelessVideoPlaybackDisabled>(connection, decoder, this, &RemoteMediaPlayerProxy::setWirelessVideoPlaybackDisabled);
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetWirelessPlaybackTarget::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetWirelessPlaybackTarget>(connection, decoder, this, &RemoteMediaPlayerProxy::setWirelessPlaybackTarget);
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetShouldPlayToPlaybackTarget::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetShouldPlayToPlaybackTarget>(connection, decoder, this, &RemoteMediaPlayerProxy::setShouldPlayToPlaybackTarget);
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetLegacyCDMSession::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetLegacyCDMSession>(connection, decoder, this, &RemoteMediaPlayerProxy::setLegacyCDMSession);
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::KeyAdded::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::KeyAdded>(connection, decoder, this, &RemoteMediaPlayerProxy::keyAdded);
#endif
#if ENABLE(ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::CdmInstanceAttached::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::CdmInstanceAttached>(connection, decoder, this, &RemoteMediaPlayerProxy::cdmInstanceAttached);
#endif
#if ENABLE(ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::CdmInstanceDetached::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::CdmInstanceDetached>(connection, decoder, this, &RemoteMediaPlayerProxy::cdmInstanceDetached);
#endif
#if ENABLE(ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::AttemptToDecryptWithInstance::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::AttemptToDecryptWithInstance>(connection, decoder, this, &RemoteMediaPlayerProxy::attemptToDecryptWithInstance);
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA) && ENABLE(ENCRYPTED_MEDIA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetShouldContinueAfterKeyNeeded::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetShouldContinueAfterKeyNeeded>(connection, decoder, this, &RemoteMediaPlayerProxy::setShouldContinueAfterKeyNeeded);
#endif
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::BeginSimulatedHDCPError::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::BeginSimulatedHDCPError>(connection, decoder, this, &RemoteMediaPlayerProxy::beginSimulatedHDCPError);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::EndSimulatedHDCPError::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::EndSimulatedHDCPError>(connection, decoder, this, &RemoteMediaPlayerProxy::endSimulatedHDCPError);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::NotifyActiveSourceBuffersChanged::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::NotifyActiveSourceBuffersChanged>(connection, decoder, this, &RemoteMediaPlayerProxy::notifyActiveSourceBuffersChanged);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::ApplicationWillResignActive::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::ApplicationWillResignActive>(connection, decoder, this, &RemoteMediaPlayerProxy::applicationWillResignActive);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::ApplicationDidBecomeActive::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::ApplicationDidBecomeActive>(connection, decoder, this, &RemoteMediaPlayerProxy::applicationDidBecomeActive);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::NotifyTrackModeChanged::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::NotifyTrackModeChanged>(connection, decoder, this, &RemoteMediaPlayerProxy::notifyTrackModeChanged);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::TracksChanged::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::TracksChanged>(connection, decoder, this, &RemoteMediaPlayerProxy::tracksChanged);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::AudioTrackSetEnabled::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::AudioTrackSetEnabled>(connection, decoder, this, &RemoteMediaPlayerProxy::audioTrackSetEnabled);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::VideoTrackSetSelected::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::VideoTrackSetSelected>(connection, decoder, this, &RemoteMediaPlayerProxy::videoTrackSetSelected);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::TextTrackSetMode::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::TextTrackSetMode>(connection, decoder, this, &RemoteMediaPlayerProxy::textTrackSetMode);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::PerformTaskAtMediaTime::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaPlayerProxy::PerformTaskAtMediaTime>(connection, decoder, this, &RemoteMediaPlayerProxy::performTaskAtMediaTime);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetVideoPlaybackMetricsUpdateInterval::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetVideoPlaybackMetricsUpdateInterval>(connection, decoder, this, &RemoteMediaPlayerProxy::setVideoPlaybackMetricsUpdateInterval);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetPreferredDynamicRangeMode::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetPreferredDynamicRangeMode>(connection, decoder, this, &RemoteMediaPlayerProxy::setPreferredDynamicRangeMode);
#if ENABLE(WEB_AUDIO)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::CreateAudioSourceProvider::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::CreateAudioSourceProvider>(connection, decoder, this, &RemoteMediaPlayerProxy::createAudioSourceProvider);
#endif
#if ENABLE(WEB_AUDIO)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::SetShouldEnableAudioSourceProvider::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::SetShouldEnableAudioSourceProvider>(connection, decoder, this, &RemoteMediaPlayerProxy::setShouldEnableAudioSourceProvider);
#endif
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::PlayAtHostTime::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::PlayAtHostTime>(connection, decoder, this, &RemoteMediaPlayerProxy::playAtHostTime);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::PauseAtHostTime::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::PauseAtHostTime>(connection, decoder, this, &RemoteMediaPlayerProxy::pauseAtHostTime);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::StartVideoFrameMetadataGathering::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::StartVideoFrameMetadataGathering>(connection, decoder, this, &RemoteMediaPlayerProxy::startVideoFrameMetadataGathering);
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::StopVideoFrameMetadataGathering::name())
        return IPC::handleMessage<Messages::RemoteMediaPlayerProxy::StopVideoFrameMetadataGathering>(connection, decoder, this, &RemoteMediaPlayerProxy::stopVideoFrameMetadataGathering);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

bool RemoteMediaPlayerProxy::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::WouldTaintOrigin::name())
        return IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerProxy::WouldTaintOrigin>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerProxy::wouldTaintOrigin);
#if PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::ErrorLog::name())
        return IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerProxy::ErrorLog>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerProxy::errorLog);
#endif
#if PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::AccessLog::name())
        return IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerProxy::AccessLog>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerProxy::accessLog);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::NativeImageForCurrentTime::name())
        return IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerProxy::NativeImageForCurrentTime>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerProxy::nativeImageForCurrentTime);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::ColorSpace::name())
        return IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerProxy::ColorSpace>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerProxy::colorSpace);
#endif
    if (decoder.messageName() == Messages::RemoteMediaPlayerProxy::VideoFrameForCurrentTimeIfChanged::name())
        return IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerProxy::VideoFrameForCurrentTimeIfChanged>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerProxy::videoFrameForCurrentTimeIfChanged);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
